/*
	Copyright 2007 Martin Gernhard

	This file is part of Jump And Run

    Jump And Run is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License Version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Liest die einzelnen Animationsphasen (Bilder und Anzeigezeiten) aus der
 * Animationsdatei und erzeugt daraus Animationen.
 * \author Martin Gernhard
 */

#include <iostream>
#include <string>
#include <fstream>
#include "utils.h"

#include "animator.h"

#include <list>

Animator::Animator(const std::string& animationsdatei, SpriteManager* spriteManager)
:modus(STEHEN), richtung(RICHTUNG_RECHTS), spiegeln(true)
{
	this->spriteManager = spriteManager;
	std::string Buffer;
	std::ifstream InFile( animationsdatei.c_str() );

	if( !InFile ){
		sdlFehlerAusgeben(animationsdatei + " nicht gefunden!");
		exit(1);
	}
	
	void* status;
	if ( (status = std::getline(InFile, Buffer)) != NULL){
		if (std::string("nomirror") == std::string(Buffer)){
			spiegeln = false;
		}
	}
	
	// Animationsphasen einlesen.
	for (int i = 0; i < ANZAHL_BEWEGUNGSMODI; i++){
		std::list<std::string> animationsliste;
		void* status;
		while (((status = std::getline(InFile, Buffer)) != NULL) && (Buffer.compare("?") != 0)){
			if (Buffer.c_str()[0] != '#'){
				animationsliste.push_back(Buffer);
			}
		}
		animationen[i] = new Animation(animationsliste, spriteManager);
		// Wurde das Dateiende erreicht, bevor alle Animationsphasen eingelesen wurden?
		if ((status == NULL) && (i != (ANZAHL_BEWEGUNGSMODI - 1))){
			sdlFehlerAusgeben(animationsdatei + ": unerwartetes Dateiende. Evtl. fehlende Animationsphasen oder korrupte Datei.");
			exit(1);
		}
	}

}

Animator::~Animator()
{
	for (int i = 0; i < ANZAHL_BEWEGUNGSMODI; i++){
		delete animationen[i];
	}
}

void Animator::setzeBewegungsmodus(Bewegungsmodus modus)
{
	if (this->modus != modus){
		this->modus = modus;
		
		for (int i = 0; i < ANZAHL_BEWEGUNGSMODI; i++){
			animationen[i]->setzeZurueck();
		}
	}
}

Sprite* Animator::erstesBild()
{
	// HACK: Allererstes Bild unabhngig vom Modus
	return (animationen[0])->erstesBild();
}

Sprite* Animator::naechstesBild()
{
	return (animationen[modus])->naechstesBild();
}

Sprite* Animator::aktuellesBild()
{
	return (animationen[modus])->aktuellesBild();
}

void Animator::setzeRichtung(Richtung richtung)
{
	this->richtung = richtung;
}

void Animator::zeichnen(int x, int y)
{
	if (spiegeln && richtung.enthaelt(RICHTUNG_LINKS)){
		aktuellesBild()->gespiegeltZeichnen(x, y, erstesBild()->getBreite());
	}else{
		aktuellesBild()->zeichnen(x, y);
	}
}
