/*
	Copyright 2007 Martin Gernhard

	This file is part of Jump And Run

    Jump And Run is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License Version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Einfache Klasse, die ein Bild einer Animationsphase speichert.
 * \author Martin Gernhard
 */

#ifndef FRAME_H
#define FRAME_H

#include <string>

#include "sprite.h"
#include "spritemanager.h"

class Frame{
	private:

	// Die Zeitspanne, fr die das Frame angezeigt werden soll, angegeben in
	// 1/50 Sekunden.
	int anzeigedauer;

	Sprite* sprite;

	public:

	/**
	 * Erstellt ein neues Frame aus einer Bilddatei.
	 * Die Anzeigedauer wird auf 1/50 Sekunde festgelegt.
	 * \param dateiname Ist der Pfad der zu benutzenden Bilddatei.
	 */
	Frame(const std::string& dateiname, SpriteManager* spriteManager);

	/**
	 * Erstellt ein neues Frame aus einer Bilddatei.
	 * \param dateiname Ist der Pfad der zu benutzenden Bilddatei.
	 * \param anzeigedauer Die Zeitspanne, fr die das Frame angezeigt
	 * werden soll, anzugeben in 1/50 Sekunden.
	 */
	Frame(const std::string& dateiname, int anzeigedauer, SpriteManager* spriteManager);


	/**
	 * \returns Die Zeitspanne, fr die das Frame angezeigt werden soll,
	 * angegeben in 1/50 Sekunden.
	 */
	int getAnzeigedauer();

	/**
	 * \returns Das zu diesem Frame gehrende Sprite;
	 */
	Sprite* getSprite();
};


#endif	// FRAME_H
