/*
	Copyright 2007 Zafir Mahmutovic, Martin Gernhard

	This file is part of Jump And Run

    Jump And Run is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License Version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "hintergrund.h"
#include <iostream>

Hintergrund::Hintergrund( std::string dateiname, Level* level, SpriteManager* spriteManager )
{
	this->level = level;
	// Hintergrund-"Sprite" initialisieren.
#ifndef OUTPUT_SDL
	grafik = spriteManager->createSprite( dateiname );
#else
	grafik = spriteManager->erzeugeSDLHintergrund( dateiname );
#endif
	verschiebung = 0;
}

Hintergrund::~Hintergrund()
{
}

void Hintergrund::setVerschiebung(int verschiebung)
{
	this->verschiebung = verschiebung / parallaxTeiler;
	
	// Hintergrund im gltigen Bereich halten:
	this->verschiebung = max(0, this->verschiebung);
	this->verschiebung = min(
				((level->getBreite() - SCREEN_WIDTH)/parallaxTeiler),
				this->verschiebung);
}

void Hintergrund::zeichnen()
{
	const int zeichenVerschiebung = verschiebung % grafik->getBreite();
#ifndef OUTPUT_SDL
	grafik->zeichnen( 0 - zeichenVerschiebung, 0 ); 
	grafik->zeichnen( grafik->getBreite() - zeichenVerschiebung, 0 );
#else
	// Erster Teil (beginnt spter)
	SDL_Rect src;
	src.x = zeichenVerschiebung;
	src.y = 0;
	src.h = grafik->getHoehe();
	src.w = grafik->getBreite() - zeichenVerschiebung - 1;

	SDL_BlitSurface(grafik->getBilddaten(), &src, mainScreen, 0);

	// Zweiter Teil	(endet frher)
	src.x = 0;
	src.y = 0;
	src.h = grafik->getHoehe();
	src.w = zeichenVerschiebung;

	SDL_Rect dest;
	dest.x = grafik->getBreite() - zeichenVerschiebung - 1 ;
	dest.y = 0;

	SDL_BlitSurface(grafik->getBilddaten(), &src, mainScreen, &dest);
#endif
}
