/*
	Copyright 2007 Zafir Mahmutovic, Martin Gernhard

	This file is part of Jump And Run

    Jump And Run is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License Version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * \author Zafir Mahmutovic
 * \author Martin Gernhard
 */

#ifndef LEVEL_H_
#define LEVEL_H_

#include <vector>
#include <list>

#include <SDL.h>

#include "utils.h"
#include "tiletyp.h"
#include "tilemanager.h"
#include "spritemanager.h"
#include "kiste.h"

class Figur;
class Tile;


class Level
{
private:

	// Dimensionen des Levels
	int levelBreite;
	int levelHoehe;
	
	int anzahlTilesX;
	int anzahlTilesY;

	// Die Kamera
	SDL_Rect camera;
	
	// Die Kamera in Tile-Koordinaten
    int cameraLeft;
    int cameraTop;
    int cameraRight;
    int cameraBottom;

	// Der Manager fr die Tile-Sprites.
	TileManager* tileManager;

    // Die Tiles selbst
	std::vector<std::vector<Tile* > > tiles;

	// Bonuskisten
	std::list<Kiste*> kisten;

	// Level aus einer Datei laden
	bool laden( const std::string& dateiname );

	bool isInvalidTile(int x, int y);
	void kameraAktualisieren();

public:
	Level( const std::string& dateiname, SpriteManager* spriteManager );
	~Level();

	void zeichnen();
	bool isEnde();
	void setVerschiebung(int verschiebung);

	bool isSolid(int TileX, int TileY);

	int getBreite(){ return levelBreite; }
	int getHoehe(){ return levelHoehe; }

	TileTyp getTileTypAnPosition(int x, int x);
	void setTileTypAnPosition(int x, int y, TileTyp typ);

	TileTyp getTileTyp(int tileX, int tileY);
	void setTileTyp(int tileX, int tileY, TileTyp typ);

	void speichern(std::string dateiname);

	void groesseAendern(int anzahlTilesX, int anzahlTilesY);

	void kisteHinzufuegen(Kiste* kiste);
	void kisteEntfernen(Kiste* kiste);
	Kiste* sucheKisteAnPosition(int tileX, int tileY);

	void neustart();
};

#endif /* LEVEL_H_ */
