/*
	Copyright 2007 Martin Gernhard

	This file is part of Jump And Run

    Jump And Run is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License Version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * \author Martin Gernhard
 */

#include <iostream>
#include <sstream>

#include "linksrechtsbewegung.h"
#include "logger.h"


LinksrechtsBewegung::LinksrechtsBewegung(Figur *figur)
:Steuerfunktion()
{
	this->figur = figur;
	this->geschwindigkeit = startGeschwindigkeit;
	figur->setDX(startGeschwindigkeit);
}

bool LinksrechtsBewegung::istBeendet()
{
	return beendet;
}

void LinksrechtsBewegung::kollision(Kollision& kollision)
{
	int dx = figur->getDX();
	
	if ( ((dx > 0) && kollision.kollidiertRechts())
		|| ((dx < 0) && kollision.kollidiertLinks() ) )
	{
		// Richtung wechseln.
		figur->setDX(0 - dx);
	}
}

std::string LinksrechtsBewegung::getName()
{
	return STEUERFUNKTION_LINKSRECHTSBEWEGUNG;
}

void LinksrechtsBewegung::setParameter(std::string parameter)
{
	logger->debug("Parameter = " + parameter);
	if (parameter != ""){
		std::istringstream parameterStream(parameter);
		parameterStream >> geschwindigkeit;
		setGeschwindigkeit(geschwindigkeit); 
	}
}

std::string LinksrechtsBewegung::alsString()
{
	std::ostringstream output;
	output << getName() << " " << geschwindigkeit;
	return output.str();
}

void LinksrechtsBewegung::setGeschwindigkeit(int geschwindigkeit)
{
	this->geschwindigkeit = geschwindigkeit;
	figur->setDX(geschwindigkeit);
}
