#include "logger.h"
#include <iostream>

Logger* logger;

std::map<std::string, int> Logger::stringLoglevelMap;

const int Logger::LOGLEVEL_DEBUG = 4;
const int Logger::LOGLEVEL_INFO  = 3;
const int Logger::LOGLEVEL_WARN  = 2;
const int Logger::LOGLEVEL_ERROR = 1;

const std::string Logger::LOGSTRING_DEBUG = "debug";
const std::string Logger::LOGSTRING_INFO  = "info";
const std::string Logger::LOGSTRING_WARN  = "warn";
const std::string Logger::LOGSTRING_ERROR = "error";

void Logger::stringLoglevelMapInit()
{
	if (stringLoglevelMap.empty()){
		stringLoglevelMap[LOGSTRING_DEBUG] = LOGLEVEL_DEBUG;
		stringLoglevelMap[LOGSTRING_INFO]  = LOGLEVEL_INFO;
		stringLoglevelMap[LOGSTRING_WARN]  = LOGLEVEL_WARN;
		stringLoglevelMap[LOGSTRING_ERROR] = LOGLEVEL_ERROR;
	}
}

void Logger::initLogging(int level)
{
	stringLoglevelMapInit();
	logger = new Logger(level);
}

void Logger::shutdownLogging()
{
	delete logger;
	logger = NULL;
}

Logger::Logger(int level)
{
	stringLoglevelMapInit();
	this->level = level;
}

int Logger::getLevel()
{
	return this->level;
}

void Logger::setLevel(int level)
{
	this->level = level;
}

void Logger::setLevel(std::string level)
{
	this->level = stringLoglevelMap[level];
}

void Logger::debug(std::string message)
{
	if (getLevel() >= Logger::LOGLEVEL_DEBUG){ 
		std::cout << message << std::endl;
	}
}
void Logger::info(std::string message)
{
	if (getLevel() >= Logger::LOGLEVEL_INFO){ 
		std::cout << message << std::endl;
	}
}
void Logger::warn(std::string message)
{
	if (getLevel() >= Logger::LOGLEVEL_WARN){ 
		std::cout << message << std::endl;
	}
}
void Logger::error(std::string message)
{
	if (getLevel() >= Logger::LOGLEVEL_ERROR){ 
		std::cout << message << std::endl;
	}
}
