/*
	Copyright 2007 Martin Gernhard, Zafir Mahmutovic, Sebastian von Helden

	This file is part of Jump And Run

    Jump And Run is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License Version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * \author Martin Gernhard
 * \author Zafir Mahmutovic
 * \author Sebastian von Helden 
 */

#include <stdlib.h>
#include <iostream>
#include <sstream>
#include <fstream>
#include <vector>

#include <SDL.h>
#include <SDL_image.h>

#include "logger.h"
#include "utils.h"
#include "sprite.h"
#include "figur.h"
#include "ring.h"
#include "status.h"
#include "punkte.h"
#include "kiste.h"
#include "herzkiste.h"
#include "bonussprung.h"

#include "spielerbewegung.h"
#include "spielerbewegungy.h"
#include "linksrechtsbewegung.h"
#include "gegnersprung.h"
#include "level.h"
#include "tile.h"
#include "hintergrund.h"
#include "spritemanager.h"
#include "bitmapfont.h"
#include "stachel.h"
#include "soundmanager.h"

bool spiel_laden(int& levelNummer)
{
	std::ifstream nummer("levelnummer.save");

	if( nummer == NULL ){
		logger->error("Der Spielstand konnte nicht geladen werden.");
		return false;    
    }     

	nummer >> levelNummer;
	nummer.close();
	logger->info("Spielstand geladen, Level-Nummer: " + itos(levelNummer));

	return true;
}

bool spiel_speichern(int levelNummer)
{
    std::ofstream nummer("levelnummer.save");
    
	if( nummer == NULL ){
		logger->error("Der Spielstand konnte nicht geschrieben werden.");
		return false;
	}

	nummer << levelNummer;
	nummer.close();
	logger->info("Spielstand gespeichert, Level-Nummer: " + itos(levelNummer));

	return true;
}

bool isInRechteck(int x, int y, SDL_Rect& rect)
{
	return (		x < rect.x + rect.w && x >= rect.x
				&&	y < rect.y + rect.h && y >= rect.y);
}

int menue(SpriteManager& spriteManager, BitmapFont& schwarz, BitmapFont& weiss,
		   BitmapFont& rot, SDL_Joystick* joystick)
{
	std::string neuText   = "Neues Spiel";
	std::string ladenText = "Spiel laden";
	std::string endeText  = "Ende";
	
	SDL_Rect neu   = {1 * 32, 1 * 32, 1 + neuText.length()   * 32, 1 * 32};
	SDL_Rect laden = {1 * 32, 4 * 32, 1 + ladenText.length() * 32, 1 * 32};
	SDL_Rect ende  = {1 * 32, 7 * 32, 1 + endeText.length()  * 32, 1 * 32};

	SDL_ShowCursor(1);
	Sprite* menueBild = spriteManager.createSprite("images/title.png");

	while(true){
		SDL_PumpEvents();
		
		SDL_Event event;
		
	    ClearScreen();
		menueBild->zeichnen( 0, (HUD_HOEHE / 2) - 1 );

		if( isInRechteck(event.motion.x, event.motion.y, neu) ){
			rot.schreibeZeile(neuText, neu.x, neu.y);
			if(SDL_BUTTON(event.motion.state)==1)
				return 0;
        }else{
			schwarz.schreibeZeile(neuText, neu.x, neu.y);        	
        }

		if( isInRechteck(event.motion.x, event.motion.y, laden) ){
			rot.schreibeZeile(ladenText, laden.x, laden.y);
			if(SDL_BUTTON(event.motion.state)==1){
				int levelNummer = 0;
				if(spiel_laden(levelNummer))                                  
				return levelNummer;
			}
		}else{
			schwarz.schreibeZeile(ladenText, laden.x, laden.y);        	
		}        

        if( isInRechteck(event.motion.x, event.motion.y, ende) ){
			rot.schreibeZeile(endeText, ende.x, ende.y);
			if(SDL_BUTTON(event.motion.state)==1)
			return -1;
		}else{
			schwarz.schreibeZeile(endeText, ende.x, ende.y);			
        }

		if ( SDL_JoystickGetButton (joystick, 0) ){
			return 0;
		}

		while(SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_KEYDOWN:
				switch( event.key.keysym.sym ){
					case SDLK_q:
					case SDLK_e:
						return -1;
						break;
					case SDLK_l:
					{
						int levelNummer = 0;
						if(spiel_laden(levelNummer))                                  
						return levelNummer;
						break;
					}
					case SDLK_n:
						return 0;
						break;
					default:
						break;
				}
				break;
			case SDL_QUIT:
				return -1;
				break;
			}
		}

		SwapBuffers();
    }
    
    return -1;
}

bool levelSpielen(std::string levelName, SDL_Joystick* joystick,
			BitmapFont& schwarz, BitmapFont& weiss, bool& isSpielEnde,
			std::string hintergrundDateiname, Status* status,
			std::string musikDateiname, SoundManager& soundManager,
			SpriteManager& spriteManager)
{
	bool geschafft = false;
	
	// Level initialisieren
	Level* level = new Level( levelName + ".map", &spriteManager );
	
	// Spielfigur initialisieren.
	Figur *spieler = new Figur("Spielfigur", "sprites/spieler.anim", level, &spriteManager);
	spieler->setzePosition(0, SCREEN_HEIGHT / 2);
	spieler->setzeAbstaende(15, 0, 5, 0);
	
	// Gegnerliste initialisieren
	std::list<Figur*> gegner = Figur::ladeListeAusDatei( levelName + ".objects", &spriteManager, level );

	// Hintergrund initialisieren
	Hintergrund* hintergrund = new Hintergrund( hintergrundDateiname, level, &spriteManager );

	// HUD Sprites laden
	Sprite* herzvoll = spriteManager.createSprite("sprites/heart.png");
	Sprite* herzhalb = spriteManager.createSprite("sprites/hearthalf.png");
	Sprite* herzleer = spriteManager.createSprite("sprites/heartempty.png");
	Sprite* gameoverschriftzug = spriteManager.createSprite("sprites/gameover2.png");
	Sprite* kopf = spriteManager.createSprite("images/meow_head.png");

	// Tne laden.
	soundManager.ladeSound(SoundManager::SOUND_SPRUNG);
	soundManager.ladeSound(SoundManager::SOUND_PLOPP);
	soundManager.ladeSound(SoundManager::SOUND_RING);
	soundManager.ladeSound(SoundManager::SOUND_GAMEOVER);
	soundManager.ladeSound(SoundManager::SOUND_AU);
	soundManager.ladeSound(SoundManager::SOUND_NEIN);
	soundManager.ladeSound(SoundManager::SOUND_SPIELENDE);

	// Hintergrundmusik laden.
	soundManager.ladeSoundtrack(musikDateiname);
	// Hintergrundmusik in Endlosschleife abspielen.
	soundManager.spieleSoundtrack(musikDateiname);

	//Versteckt den Maus Cursor
    SDL_ShowCursor(0);
    
	// X-Bewegung des Spielers	
	Spielerbewegung* spielerbew = new Spielerbewegung(spieler);
	spieler->steuerfunktionHinzufuegen(spielerbew);
	
	// Y-Bewegung des Spielers
	SpielerbewegungY* spielerbewy = new SpielerbewegungY(spieler);
	spieler->steuerfunktionHinzufuegen(spielerbewy);

	// Hauptschleife, fragt die Tastatur, ndert ggf. die
	// Bewegungsrichtung und fhrt smtliche Scroll- und
	// Kollisionstests durch.
	
	bool running = true;
	bool gameover = false;
	bool neustart = false;
	
	while(running) {
		int startticks = SDL_GetTicks();
		
		bool sprungBeginn = false;
		bool rennEnde = false;

		// Tasten, bei denen der nur Zustand abgefragt werden soll.
		Uint8* keystates = SDL_GetKeyState(NULL);
		if (keystates[SDLK_RIGHT] || keystates[SDLK_d] ||  SDL_JoystickGetAxis (joystick, 0) > 0){
			spielerbew->setDX(maxDx);
		}else if (keystates[SDLK_LEFT] || keystates[SDLK_a] ||  SDL_JoystickGetAxis (joystick, 0) < 0){
			spielerbew->setDX(0 - maxDx);
		}else{
			spielerbew->stopX();			
		}
		
		if (keystates[SDLK_k] || keystates[SDLK_LCTRL] || SDL_JoystickGetButton (joystick, 0)){
             if(!spieler->faellt() && !spielerbewy->springt()) {
             	spieler->setRennt(true);
				spielerbew->setRUN(true);
				spielerbewy->setRUN(true);
             }
		}
			
		
		// Tasten, bei denen Drcken und Loslassen als gretrennte Ereignisse
		// behandelt werden.
		SDL_Event event;
		while(SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_KEYDOWN:
				switch( event.key.keysym.sym ){
					case SDLK_q:
						running = false;
						isSpielEnde = true;
						break;
					case SDLK_m:
						logger->debug("-------------- Markierung ------------");
						break;
					case SDLK_w:
					case SDLK_SPACE:
						sprungBeginn = true;
						break;
					default:
						break;
				}
				break;
			case SDL_KEYUP:
				switch( event.key.keysym.sym ){
					case SDLK_k:
					case SDLK_LCTRL:
						rennEnde = true;
						break;
                    default:
						break;
				}
				break;
			case SDL_JOYBUTTONUP:
				switch( event.jbutton.button ){
					case 0:
						rennEnde = true;
						break;
					default:
						break;
				}
				break;
			case SDL_JOYBUTTONDOWN:
				switch( event.jbutton.button ){
					case 1:
						sprungBeginn = true;
						break;
					default:
						break;
				}
				break;
			case SDL_QUIT:
				running = false;
				isSpielEnde = true;
				break;
			}
		}

		if (sprungBeginn){
			if (!spielerbewy->springt() && !spieler->faellt()){
				spielerbewy->springen(16);
				soundManager.spieleSound(SoundManager::SOUND_SPRUNG);
			}
		}

		if (rennEnde){
			spieler->setRennt(false);
			spielerbew->setRUN(false);
			spielerbewy->setRUN(false);
		}

		// Neue Koordinaten der Spielfigur(en) berechnen.
		spieler->naechsterZustand();

		// Testen, ob die Spielfigur unten aus dem Bild gefallen ist.
		if (spieler->isUntenRausgefallen()){
			status->setEnergieLeer();
		}


		// Verschiebung des sichtbaren Ausschnitts neu berechnen...
		int verschiebung = spieler->getX() - (SCREEN_WIDTH/2) + (spieler->getBreite()/2);
		verschiebung = max(verschiebung, 0);

        // ...und an alle Interessierten bergeben.		
		level->setVerschiebung( verschiebung );
		spieler->setVerschiebung( verschiebung );
		hintergrund->setVerschiebung( verschiebung );

        // FIXME: Levelende wirklich nur rechter Levelrand?    
		if ( spieler->getX() >  level->getBreite() - spieler->getBreite()){
			running = false;
			geschafft = true;
		} 

        /* Das Bild neu aufbauen. */
		ClearScreen();

		hintergrund->zeichnen();
        level->zeichnen();
		spieler->zeichnen();


		for ( std::list<Figur*>::iterator iter = gegner.begin(); iter != gegner.end(); iter++){
			Figur* g = *iter;
			g->naechsterZustand();
			g->setVerschiebung( verschiebung );
			g->zeichnen();			
		}

		/* HUD */

		for ( int i = 0; i < Status::ANZAHL_HERZEN; i++ ){
			if ( status->isHerzVoll(i) ){
				herzvoll->zeichnen(i * 32, 480);
			}else if( status->isHerzHalb(i) ){
				herzhalb->zeichnen(i * 32, 480);
			}else{
				herzleer->zeichnen(i * 32, 480);
			}
		}

		kopf->zeichnen(4 * 32, 480);

		std::stringstream textLeben;
		textLeben << "x" << status->getLeben();
		weiss.schreibeText(textLeben.str(), 5 * 32, 480);
		
		std::stringstream textPunkte;
		textPunkte << "Score:" << status->getPunkte();
		weiss.schreibeZeile(textPunkte.str(), 9 * 32, 480);


		/* Kollisionsabfrage mit Gegnern, Ringen etc. */

		for ( std::list<Figur*>::iterator iter = gegner.begin(); iter != gegner.end();){
			Figur* g = *iter;
			Kollision kollision = spieler->pruefeKollision(*g);

			switch(g->getTyp().getEnum()){
				case FigurTyp::FIGURTYP_STANDARD:	
					if (kollision.kollidiertMitGegner()){
						if (kollision.kollidiertUnten()){
							g->setTot(true);
							soundManager.spieleSound(SoundManager::SOUND_PLOPP);
							spielerbewy->gegner_bounce();
							status->add_punkte(g->getPunkte());
						}else{
							if (!spieler->isUnverwundbar()){
								status->sub_energie(g->getSchaden());
								soundManager.spieleSound(SoundManager::SOUND_AU);
								spieler->setzeUnverwundbar(100);
							}
						}
					}
					break;
              /*case FigurTyp::FIGURTYP_KISTE:	
					if (kollision.kollidiertMitGegner()){
						if (kollision.kollidiertOben() && spielerbewy->springt()){
							g->setTot(true);
							level->kisteEntfernen( (Kiste*)g );
							soundManager.spieleSound(SoundManager::SOUND_PLOPP);
						}
					}
					break;*/
				case FigurTyp::FIGURTYP_RING:
					if (kollision.kollidiertMitGegner()){
						g->setTot(true);
						soundManager.spieleSound(SoundManager::SOUND_RING);
						status->add_punkte(Punkte::PUNKTE_RING);
					}
					break;
				case FigurTyp::FIGURTYP_STACHEL:
					if (kollision.kollidiertMitGegner()){
						if (!spieler->isUnverwundbar()){
							status->sub_energie(g->getSchaden());
							soundManager.spieleSound(SoundManager::SOUND_AU);
							spieler->setzeUnverwundbar(100);
							
							if (kollision.kollidiertUnten()){
								spielerbewy->gegner_bounce();
							}
						}
					}
					break;
				default:
					break;
			}
			
			if(g->isTot()){
				iter = gegner.erase(iter);
				delete g;				
			}else{
				iter++;
			}
			
		}

		/* Alle berhrten Bonuskisten behandeln. */
		if (spielerbewy->springt()){
			for (std::list<Figur*>::iterator iter = spieler->getBeruehrteKisten().begin();
						iter != spieler->getBeruehrteKisten().end(); iter++)
			{
				Kiste* kiste = (Kiste*)*iter;
				
				switch(kiste->getTyp().getEnum()){
					case FigurTyp::FIGURTYP_KISTE:
					{
						if (!kiste->isEndzustand()){
							Figur* figur = new Figur("Herz", "sprites/ring.anim",
										level, &spriteManager);
							figur->steuerfunktionHinzufuegen(new BonusSprung(figur));
							figur->setzePosition(kiste->getX() + ((kiste->getBreite() - figur->getBreite()) / 2),
										kiste->getY() - figur->getHoehe());
							gegner.push_back(figur);
						}
						break;
					}
					case FigurTyp::FIGURTYP_HERZKISTE:
					{
						if (!kiste->isEndzustand()){
							status->add_energie(((HerzKiste*)kiste)->getEnergie());
							Figur* figur = new Figur("Herz", "sprites/herz.anim",
										level, &spriteManager);
							figur->steuerfunktionHinzufuegen(new BonusSprung(figur));
							figur->setzePosition(kiste->getX() + ((kiste->getBreite() - figur->getBreite()) / 2),
										kiste->getY() - figur->getHoehe());
							gegner.push_back(figur);
						}
						break;
					}
					default:
						break;
				}

				soundManager.spieleSound(kiste->getSound());
				status->add_punkte(kiste->getPunkte());
				kiste->ausgeloest();
			}
		}


		// Prfen, ob der Spieler ein Leben verloren hat.

		if(status->getEnergie() <= 0){
			status->sub_leben(1); 
			
			if(status->getLeben() > 0){
				neustart = true;
			}else{
				gameover = true;
			}
		}

		/* Prfen, ob das Spiel durch "Game Over" (Verlust aller Leben)
		 * verloren wurde und ggf. einen Schriftzug anzeigen und die
		 * Game-Over-Melodie abspielen.
		 */
		if (gameover){
			soundManager.stoppeSoundtrack();
			gameoverschriftzug->zeichnen(640/2 - 455/2, 480/2 - 127/2);
			SwapBuffers();
			soundManager.spieleSound(SoundManager::SOUND_NEIN);
			SDL_Delay(1000);
			soundManager.spieleSound(SoundManager::SOUND_GAMEOVER);
			SDL_Delay(3600);
			running = false;
			geschafft = false;
			spiel_speichern(0);
		}
		
		if (neustart){
			SwapBuffers();
			status->setEnergieVoll();
			spieler->setzePosition(0, SCREEN_HEIGHT / 2);
			
			// Verbliebene Figuren lschen.
			for (std::list<Figur*>::iterator iter = gegner.begin(); iter != gegner.end();){
				Figur* g = *iter;
				iter = gegner.erase(iter);
				delete g;			
			}

			level->neustart();

			// Figuren neu laden.
			gegner = Figur::ladeListeAusDatei( levelName + ".objects",
						&spriteManager, level );

			spieler->setzeUnverwundbar(0);
			neustart = false;
			
			soundManager.spieleSound(SoundManager::SOUND_NEIN);

			SDL_Delay(1000);
		}
		

		/* Das Bild ist jetzt aufgebaut. */
         
		// Pageflip: Back- und Frontbuffer werden vertauscht
		SwapBuffers();

		// Konstante Bildrate von 50fps.
		int ticksleft = (1000/50) - (SDL_GetTicks() - startticks);
		if (ticksleft > 0){ SDL_Delay(ticksleft); }
	}

	logger->debug("<spieler>");
	logger->debug(spieler->alsString()	+ "<spieler/>");

	// Aufrumen.
	delete spieler;
	delete hintergrund;

	// Verbliebene Figuren lschen.
	for (std::list<Figur*>::iterator iter = gegner.begin(); iter != gegner.end();){
		Figur* g = *iter;
		iter = gegner.erase(iter);
		delete g;			
	}

	delete level;

	return geschafft;
}

int main(int argc, char **argv)
{
	Logger::initLogging(Logger::LOGLEVEL_WARN);

	// Kommandozeilenparameter verarbeiten
	for (int i = 1; i < argc; i++){
		std::string parameter(argv[i]);

		if (parameter == "-loglevel"){
			if (i + 1 >= argc){
				logger->error("Kein Loglevel angegeben.");
				exit(1);
			}else{
				i++;
				std::string level(argv[i]);
				logger->setLevel(level);
				logger->info("Loglevel gesetzt auf '" + level + "'");
			}
		}
	} 

    
    SDL_OpengGL_Init();

	SDL_Joystick *joystick = NULL;
	if (SDL_NumJoysticks() > 0){
		logger->info("Joystick gefunden.");
		joystick = SDL_JoystickOpen(0);
	}

	// Levelliste aufbauen
	std::vector<std::string> levelListe = ladeStringVectorAusDatei
				("level/levelliste.txt", true);

	// Hintergrundliste aufbauen
	std::vector<std::string> hintergrundListe = ladeStringVectorAusDatei
				("level/hintergrundliste.txt", true);

	// Musikliste aufbauen
	std::vector<std::string> musikListe = ladeStringVectorAusDatei
				("level/musikliste.txt", true);

	// Global genutzte Manager initialisieren
	SpriteManager spriteManager;
	SoundManager soundManager;

	Sprite* spielendeschriftzug = spriteManager.createSprite("sprites/gameover2.png");

	// Fonts laden
	BitmapFont schwarz(&spriteManager, "fonts/bitstreamverasansblack.png");
	BitmapFont weiss(&spriteManager, "fonts/bitstreamverasanswhite.png");
	BitmapFont rot(&spriteManager, "fonts/bitstreamverasansred.png");
	
	//Title Bar
	SDL_WM_SetCaption("Jump'n Run", "Jump'n Run");

	Status *status = new Status();

	bool isSpielEnde = false;
	bool levelGeschafft = false;
	bool ende = false;

	std::vector<std::string>::iterator levelIterator = levelListe.begin();
	std::vector<std::string>::iterator hintergrundIterator = hintergrundListe.begin();
	std::vector<std::string>::iterator musikIterator = musikListe.begin();

	do{
		if (!levelGeschafft || ende){
			ende = false;
			// Warten, bis eine Auswahl getroffen wurde.
			int levelNummer = menue(spriteManager, schwarz, weiss, rot, joystick);

			isSpielEnde = (levelNummer == -1);

			levelIterator = levelListe.begin() + levelNummer;
			// Aktuellen Level im gltigen Bereich halten:
			if (levelIterator >= levelListe.end()){
				levelIterator = levelListe.begin();
			}
		
			hintergrundIterator = hintergrundListe.begin() + (levelIterator - levelListe.begin());
			musikIterator = musikListe.begin() + (levelIterator - levelListe.begin());

			delete status;
			status = new Status();
		}

		if (!isSpielEnde){
			levelGeschafft = levelSpielen(*(levelIterator), joystick, schwarz, weiss,
						isSpielEnde, *(hintergrundIterator), status,
						*(musikIterator), soundManager, spriteManager);
			
			if (levelGeschafft){
				// Jeweils nchsten Eintrag fr Level, Hintergrundbild und
				// Hintergrundmusik laden

				levelIterator++;
				hintergrundIterator++;
				musikIterator++;

				if (levelIterator == levelListe.end()){
					// Alle Level geschafft!
					soundManager.stoppeSoundtrack();
					spielendeschriftzug->zeichnen(640/2 - 455/2, 480/2 - 127/2);
					SwapBuffers();
					soundManager.spieleSound(SoundManager::SOUND_SPIELENDE);
					SDL_Delay(5000);               
					ende = true;
					spiel_speichern(0);

					levelIterator = levelListe.begin();
					hintergrundIterator = hintergrundListe.begin();
					musikIterator = musikListe.begin();
				}

				// Hintergrnde drfen sich zyklisch wiederholen
				if (hintergrundIterator == hintergrundListe.end()){
					hintergrundIterator = hintergrundListe.begin();
				}

				// Musik darf sich zyklisch wiederholen
				if (musikIterator == musikListe.end()){
					musikIterator = musikListe.begin();
				}

				spiel_speichern(levelIterator - levelListe.begin());
			}
		}

        if (!levelGeschafft){
			levelIterator = levelListe.begin();
			hintergrundIterator = hintergrundListe.begin();
			musikIterator = musikListe.begin();
		}

	}while(!isSpielEnde);

	delete status;

	Logger::shutdownLogging();
	return 0;
}
