/*
	Copyright 2007 Martin Gernhard

	This file is part of Jump And Run

    Jump And Run is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License Version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * \author Martin Gernhard
 */

#include <iostream>

#include "soundmanager.h"
#include "logger.h"

#include "utils.h"

const std::string SoundManager::SOUND_SPRUNG = "sounds/jump.ogg";
const std::string SoundManager::SOUND_RING = "sounds/ring.ogg";
const std::string SoundManager::SOUND_GAMEOVER = "sounds/gameover.ogg";
const std::string SoundManager::SOUND_PLOPP = "sounds/squish.ogg";
const std::string SoundManager::SOUND_AU = "sounds/au.ogg";
const std::string SoundManager::SOUND_NEIN = "sounds/nein.ogg";
const std::string SoundManager::SOUND_SPIELENDE = "sounds/spielende.ogg";
const std::string SoundManager::SOUND_HERZ = "sounds/herz.ogg";

const std::string SoundManager::SOUNDTRACK_OPENSOURCE = "sounds/opensource.ogg";
const std::string SoundManager::SOUNDTRACK_ARTIFICIALRECREATION = "sounds/artificialrecreation.ogg";
const std::string SoundManager::SOUNDTRACK_MONKEYBONES = "sounds/monkeybones.ogg";
const std::string SoundManager::SOUNDTRACK_SPOOKY = "sounds/spooky.ogg";

SoundManager::SoundManager()
{
}

SoundManager::~SoundManager()
{
	for (std::map<std::string, Mix_Chunk*>::iterator iter = sounds.begin(); iter != sounds.end(); iter++){
		Mix_Chunk* sound = (*iter).second;
		logger->debug("Sound " + (*iter).first + " wird entfernt.");
		delete sound;
	}
	sounds.clear();
	
	for (std::map<std::string, Mix_Music*>::iterator iter = tracks.begin(); iter != tracks.end(); iter++){
		Mix_Music* track = (*iter).second;
		logger->debug("Soundtrack " + (*iter).first + " wird entfernt.");
		Mix_FreeMusic( track );
	}
	tracks.clear();
}

Mix_Chunk* SoundManager::ladeSound(std::string dateiname)
{
	Mix_Chunk* sound = NULL;
	
	if (sounds.count(dateiname) > 0){
		// Sound ist bereits geladen, Zeiger darauf zurckgeben. 
		sound = sounds[dateiname];
	}else{
		// Sound ist noch nicht geladen, also laden wir ihn.
		logger->debug("Sound " + dateiname + " wird geladen.");
		sound = Mix_LoadWAV(dateiname.c_str());
		if ( sound == NULL){
			sdlFehlerAusgeben("Sounddatei " + dateiname + " konnte nicht geladen werden.");
			exit(1);
		}
		sounds[dateiname] = sound;
	}
	
	return sound;
}

void SoundManager::spieleSound(std::string dateiname)
{
	if (dateiname == ""){
		return;
	}

	Mix_PlayChannel(-1, ladeSound(dateiname), 0);
}

Mix_Music* SoundManager::ladeSoundtrack(std::string dateiname)
{
	Mix_Music* track = NULL;
	
	if (tracks.count(dateiname) > 0){
		// Soundtrack ist bereits geladen, Zeiger darauf zurckgeben. 
		track = tracks[dateiname];
	}else{
		// Soundtrack ist noch nicht geladen, also laden wir ihn.
		logger->debug("Soundtrack " + dateiname + " wird geladen.");
		track = Mix_LoadMUS(dateiname.c_str());
		if ( track == NULL){
			sdlFehlerAusgeben("Sounddatei " + dateiname + " konnte nicht geladen werden.");
			exit(1);
		}
		tracks[dateiname] = track;
	}
	
	return track;
}

void SoundManager::spieleSoundtrack(std::string dateiname)
{
	Mix_PlayMusic( ladeSoundtrack(dateiname), -1 );
}

void SoundManager::stoppeSoundtrack()
{
	Mix_HaltMusic();
}
