/*
	Copyright 2007 Martin Gernhard

	This file is part of Jump And Run

    Jump And Run is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License Version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * \author Martin Gernhard
 */

#ifndef SOUNDMANAGER_H_
#define SOUNDMANAGER_H_

#include <map>

#include <SDL_mixer.h>

class SoundManager
{

private:
	// Speichert einen Sound zu jedem Dateinamen
	std::map<std::string, Mix_Chunk*> sounds;
	
	// Speichert einen Soundtrack zu jedem Dateinamen
	std::map<std::string, Mix_Music*> tracks;

public:
	static const std::string SOUND_SPRUNG;
	static const std::string SOUND_RING;
	static const std::string SOUND_GAMEOVER;
	static const std::string SOUND_AU;
	static const std::string SOUND_PLOPP;
	static const std::string SOUND_NEIN;
	static const std::string SOUND_SPIELENDE;
	static const std::string SOUND_HERZ;
	
	static const std::string SOUNDTRACK_OPENSOURCE;
	static const std::string SOUNDTRACK_ARTIFICIALRECREATION;
	static const std::string SOUNDTRACK_MONKEYBONES;
	static const std::string SOUNDTRACK_SPOOKY;

	SoundManager();
	virtual ~SoundManager();
	
	Mix_Chunk* ladeSound(std::string dateiname);
	void spieleSound(std::string dateiname);

	Mix_Music* ladeSoundtrack(std::string dateiname);
	void spieleSoundtrack(std::string dateiname);
	void stoppeSoundtrack();
};

#endif /*SOUNDMANAGER_H_*/
