/*
	Copyright 2007 Martin Gernhard

	This file is part of Jump And Run

    Jump And Run is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License Version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Einfache Klasse, die eine Bilddatei in eine passende OpenGL-Textur ldt
 * und Befehle bereitstellt, um diese auf dem Bildschirm zu zeichnen.
 * \author Martin Gernhard
 */

#ifndef SPRITE_H
#define SPRITE_H

#include <SDL.h>

#ifndef OUTPUT_SDL
	#include <SDL_opengl.h>
#endif	

#include <string>

class SpriteManager;

class Sprite{
	friend class SpriteManager;
	
	private:

	// Die Breite der verwendeteten Grafik
	int breite;
	// Die Hhe der verwendeteten Grafik
	int hoehe;

	// Die Breite der verwendeteten Textur
	int texb;
	// Die Hhe der verwendeteten Textur
	int texh;

#ifndef OUTPUT_SDL
	// Referenznummer der Textur
	GLuint textur;
#endif

	// Bilddaten, unabhngig von der Textur. Wird z.B. fr Kollisionsabfrage
	// genutzt.
	SDL_Surface* bilddaten;

	#ifdef OUTPUT_SDL
	SDL_Surface* spiegelBilddaten;
	void erzeugeSpiegelBilddaten();
	#endif
	
	/**
	 * Erstellt ein neues Sprite aus einer Bilddatei.
	 * \param dateiname Ist der Pfad der zu benutzenden Bilddatei.
	 */
	Sprite(const std::string& dateiname);
	
	Sprite(	SDL_Surface* bilddaten );


#ifndef OUTPUT_SDL
	void erzeugeTexturAusSurface();
#endif	

	void ladeSurface(const std::string& dateiname);

	public:

	// Gibt die verwendete Textur und SDL_Surface frei.
	~Sprite();

	/**
	 * Zeichnet das Sprite auf den Bildschirm.
	 * \param x Die X-Position, an der die linke obere Ecke des Sprites
	 * gezeichnet werden soll.
	 * \param y Die X-Position, an der die linke obere Ecke des Sprites
	 * gezeichnet werden soll.
	 */
	void zeichnen(int x, int y);

#ifndef OUTPUT_SDL
	/**
	 * Zeichnet das Sprite auf den Bildschirm.
	 * \param x Die X-Position, an der die linke obere Ecke des Sprites
	 * gezeichnet werden soll.
	 * \param y Die X-Position, an der die linke obere Ecke des Sprites
	 * gezeichnet werden soll.
	 * \param breite Die Breite, mit der das Sprite gezeichnet werden soll.
	 * \param hoehe Die Hoehe, mit der das Sprite gezeichnet werden soll.
	 */
	void zeichnen(int x, int y, int breite, int hoehe);
#endif
	
	void gespiegeltZeichnen(int x, int y);
	
	void gespiegeltZeichnen(int x, int y, int breite);

	/**
	 * \returns Die Breite der Grafik
	 */
	int getBreite();
	
	/**
	 * \returns Die Hoehe der Grafik
	 */
	int getHoehe();
	
	/**
	 * \returns Die Bilddaten der Grafik
	 */
	SDL_Surface* getBilddaten();

};


#endif	// SPRITE_H
