/*
	Copyright 2007 Martin Gernhard

	This file is part of Jump And Run

    Jump And Run is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License Version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Superklasse fr alle Klassen, die Bewegungs- oder Steuerungsfunktionen
 * fr Spielfiguren implementieren
 * \author Martin Gernhard
 */

#include <iostream>
#include <sstream>

#include "steuerfunktion.h"
#include "logger.h"

#include "linksrechtsbewegung.h"
#include "gegnersprung.h"
#include "bonussprung.h"

const std::string Steuerfunktion::STEUERFUNKTION_GEGNERSPRUNG        = "Gegnersprung";
const std::string Steuerfunktion::STEUERFUNKTION_LINKSRECHTSBEWEGUNG = "LinksrechtsBewegung";
const std::string Steuerfunktion::STEUERFUNKTION_BONUSSPRUNG = "BonusSprung";

Steuerfunktion::Steuerfunktion()
{
	this->beendet = false;
}

void Steuerfunktion::ausfuehren()
{
}

bool Steuerfunktion::istBeendet()
{
	return false;
}

void Steuerfunktion::kollision(Kollision& kollision)
{
}

std::string Steuerfunktion::getName()
{
	return "Unbenannte Steuerfunktion";
}

std::string Steuerfunktion::alsString()
{
	return getName();
}

Steuerfunktion* Steuerfunktion::ausString(Figur* figur, std::string input_)
{
	std::stringstream input(input_);

	std::string typ;
	input >> typ;

	logger->debug("STF = '" + typ + "'");

	Steuerfunktion* stf;

	if (typ == STEUERFUNKTION_LINKSRECHTSBEWEGUNG){
		stf = new LinksrechtsBewegung(figur);
	}else if (typ == STEUERFUNKTION_GEGNERSPRUNG){
		stf = new Gegnersprung(figur);
	}else if (typ == STEUERFUNKTION_BONUSSPRUNG){
		stf = new BonusSprung(figur);
	}else{
		throw "Unbekannte Steuerfunktion";
	}

	std::string rest;
	getline(input, rest);
	stf->setParameter(rest);

	return stf;
}

void Steuerfunktion::setParameter(std::string parameter)
{
}

std::string Steuerfunktion::getParameter()
{
	return "";
}
