/*
	Copyright 2007 Martin Gernhard

	This file is part of Jump And Run

    Jump And Run is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License Version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * \author Martin Gernhard
 */


#ifndef TILE_H_
#define TILE_H_

#include "sprite.h"
#include "tiletyp.h"

class Tile
{
	friend class TileManager;
	
private:
	// Position des Tiles auf der Karte.
	int x;
	int y;
	
	// Das Sprite mit dem Bild des Tiles
	Sprite* sprite;
	
	// Ist das Tile undurchdringlich?
	bool solid;
	
	// Der Type des Tiles.
	TileTyp typ;
	
	static const std::string typNamen[];
	Tile(int x, int y, bool solid, TileTyp typ, Sprite* sprite);
	
public:
	// Dimensionen der Tiles
	static const int TILE_BREITE = 32;
	static const int TILE_HOEHE = 32;

	int getX();
	void setX(int x);
	
	int getY();
	void setY(int y);
	
	void zeichnen(int x, int y);
	void zeichnen();

	bool isSolid();
	
	TileTyp getTileTyp();
	
	std::string getTypName();	
	static std::string getTypName(int typ);
};

#endif /*TILE_H_*/
