/*
	Copyright 2007 Martin Gernhard

	This file is part of Jump And Run

    Jump And Run is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License Version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/** Sorgt dafr, dass alle Tile-Sprites nur einmal geladen werden.
 * \author Martin Gernhard
 */

#include <iostream>
#include <utility>

#include "tilemanager.h"

typedef std::pair<std::string, bool> EigenschaftenPair;

TileManager::TileManager(SpriteManager* spriteManager)
{
	this->spriteManager = spriteManager;
	eigenschaftenInit();
}

TileManager::~TileManager()
{
}


Tile* TileManager::createTile(int x, int y, TileTyp typ)
{
	if (typ == TILETYP_LEER){
		return NULL;
	}

	EigenschaftenPair pair = eigenschaften[typ];
	Sprite* sprite = spriteManager->createSprite(pair.first);
	
	return new Tile(x, y, pair.second, typ, sprite);
}

std::map<TileTyp, std::pair<std::string, bool> > TileManager::eigenschaften;

void TileManager::eigenschaftenInit()
{
	if (eigenschaften.empty()){
		eigenschaften[TILETYP_LEER]   = EigenschaftenPair("", false);
		eigenschaften[TILETYP_BLOCK]  = EigenschaftenPair("tiles/block1.png", true);
		eigenschaften[TILETYP_GRASS]  = EigenschaftenPair("tiles/grass1.png", false);
		eigenschaften[TILETYP_GRASS2] = EigenschaftenPair("tiles/grass2.png", false);
//		eigenschaften[TILETYP_FRAGEZEICHEN_BLOCK] = EigenschaftenPair("tiles/block2.png", true);
	}
}
