/*
	Copyright 2007 Martin Gernhard

	This file is part of Jump And Run

    Jump And Run is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License Version 2 as
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 * Hilfsfunktionen.
 * \author Martin Gernhard
 */

#ifndef UTILS_H
#define UTILS_H

#include <string>
#include <vector>

#include <SDL.h>

#ifndef OUTPUT_SDL
	#include <SDL_opengl.h>
#endif

// TODO: auslagern von files, glutils etc.

int naechsteZweierpotenz(int num);
void sdlFehlerAusgeben(const std::string &meldung);

#ifndef OUTPUT_SDL
	void GL_TexEnable(int enable);
	void GL_Bind(GLuint texture);
#endif

// Byteordnung fr PNG-Grafiken
#if SDL_BYTEORDER == SDL_BIG_ENDIAN
const Uint32 rmask = 0xff000000;
const Uint32 gmask = 0x00ff0000;
const Uint32 bmask = 0x0000ff00;
const Uint32 amask = 0x000000ff;
#else
const Uint32 rmask = 0x000000ff;
const Uint32 gmask = 0x0000ff00;
const Uint32 bmask = 0x00ff0000;
const Uint32 amask = 0xff000000;
#endif


/**
 * Einfache Hilfsfunktion, die den Greren von zwei Werten zurckgibt.
 */
inline int max(int a, int b)
{
	return (a > b) ? a : b;
}

/**
 * Einfache Hilfsfunktion, die den Kleineren von zwei Werten zurckgibt.
 */
inline int min(int a, int b)
{
	return (a < b) ? a : b;
}


// Fenstergre
static const int SCREEN_HEIGHT = 480;
static const int SCREEN_WIDTH = 640;

// Maximale Bewegung der Spielfigur.
const int maxDx = 4;
const int maxDy = 4;

// Gravitationskraft
static const int GRAVITATION = 1;

extern SDL_Surface* mainScreen;

// Haupt-Initialisierungsroutine;
SDL_Surface* SDL_OpengGL_Init();

static const int HUD_HOEHE = 32;

std::vector<std::string> parse(std::string input_, int anzahl);

#ifndef OUTPUT_SDL
	#define SwapBuffers() SDL_GL_SwapBuffers()
	#define ClearScreen() glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT)
#else
	#define SwapBuffers() SDL_Flip(mainScreen)
	#define ClearScreen() SDL_FillRect(mainScreen, NULL, SDL_MapRGB(mainScreen->format, 0, 0, 0))
#endif

void zeichneRechteck(int x, int y, int breite, int hoehe);
void zeichneHohlRechteck(int x, int y, int breite, int hoehe);

void trim(std::string& str);

std::vector<std::string> ladeStringVectorAusDatei(std::string dateiname,
			bool leereZeilenIgnorieren);

std::string itos(int wert);

/*
std::string operator+(std::string string, int integer)
{
	std::string output(string);
	output += intToString(integer);
	return output;
}

std::string operator+(int integer, std::string string)
{
	std::string output;
	output = intToString(integer) + string;
	return output;
}
*/
#endif
